#!/usr/bin/perl

################################################################################
# ImportOSM2sqlite Version 0.0.1  2011-07-19                                   #
# Written by Klaus Engelbertz                                                  #
# Copyright (C) 2011 Klaus Engelbertz                                          #
#                                                                              #
# This program is free software: you can redistribute it and/or modify         #
# it under the terms of the GNU General Public License as published by         #
# the Free Software Foundation, either version 3 of the License.               #
# This program is distributed in the hope that it will be useful,              #
# but WITHOUT ANY WARRANTY; without even the implied warranty of               #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                         #  
# See the GNU General Public License for more details.                         #
# http://www.gnu.org/licenses/                                                 #
################################################################################

use utf8;
use IO::File;
####
# Nachfolgende Module aus dem Unterverzeichnis /Modules laden
####
use lib Modules;
use DatabaseCreator 0.0.2;
use DataSetNodes 0.0.2;
use DataSetWays 0.0.2;
use DataSetRel 0.0.2;

$Databasename = 'osm.sqlitedb';
$VERSION = '0.0.2';

$TagArt = 0; # 1 = Node, 2 = Way, 3 = Ref, 4 = Relation

print "ImportOSM2sqlite $VERSION Copyright (C) 2011 by Klaus Engelbertz\n";
print "This program comes with ABSOLUTELY NO WARRANTY!\n\n";

if ($#ARGV == -1) 
{
  print "ImportOSM2sqlite myOSMfile.osm\n\n";
  exit;
}
    
$source = IO::File->new( $ARGV[0], 'r' );
binmode( $source, ':utf8' );
$DBC = DatabaseCreator->new($Databasename);# Modul zum anlegen von Datenbanken
$DSN = DataSetNodes->new($Databasename);   # Modul fuer Bearbeitung der Node-Tags - muss als erstes Modul aus diesem Paket geladen werden
$DSW = DataSetWays->new($Databasename);    # Modul fuer Bearbeitung der Way-Tags
$DSR = DataSetRel->new($Databasename);     # Modul fuer Bearbeitung der Relation-Tags 

@statDatei = stat($ARGV[0]);  # Groesse der zu lesenden Datei ermitteln fuer Fortschrittsanzeige 
$TotalBytes   = $statDatei[7];# Groesse der zu lesenden Datei in Variable schreiben     
$Abgearbeitet = 0;            # Fortschrittsanzeige, Wert der bereits abgearbeiteten Bytes
$Zeilencount  = 0;            # Zaehler fuer bereits abgearbeitete Zeilen

####
# Das Dateiformat aendert sich immer mal wieder - wir muessen die Version pruefen
# damit es keine boese Ueberraschung gibt 
####
PruefeAPIVersion(); # Nur API Version 0.6 wird akzeptiert. Beenden, wenn dies nicht verifiziert werden kann
print "Korrekte Version gefunden. Beginne Datenimport.\n";
while (<$source>)
{
####
# Zeile zuweisen
####
	$Zeile = $_;
####
# Fuer die Fortschrittsanzeige die verarbeiteten Bytes zaehlen
####	
	$Abgearbeitet += length($Zeile);
####
# Bestimmte Tags wollen wir nicht in der Datenbank haben
####	
	next if ($Zeile =~ /<tag k=\"created_by\"/);
#	next if ($Zeile =~ /<tag k=\"name:ru\"/);
#	next if ($Zeile =~ /<tag k=\"ele\"/);
#	next if ($Zeile =~ /<tag k=\"note\"/);
	next if ($Zeile =~ /<tag k=\"source\"/);
#	next if ($Zeile =~ /<tag k=\"FIXME.*\"/i);
	next if ($Zeile =~ /<tag k=\"converted_by\"/i);
	next if ($Zeile =~ /<tag k=\"url.*\"/i);
	next if ($Zeile =~ /<\/osm>/i);
	$ret = 0;
####
# TagArt zuweisen, wenn moeglich
####
  if ($Zeile =~ /<node /)      { $TagArt = 1; }
  if ($Zeile =~ /<way /)       { $TagArt = 2; }
  if ($Zeile =~ /<ref /)       { $TagArt = 3; }
  if ($Zeile =~ /<relation /)  { $TagArt = 4; }
  
  if ($TagArt == 1) # Node Tag
  { 
  	$ret = $DSN->CrunchLine($Zeile);
  	if ($ret ==  0) { print "Nicht verarbeitete Zeile: $Zeile"; }
  	if ($ret == 99) { $TagArt = 0; } # 99 => Datensatz wurde in die Datenbank geschrieben, bereit fuer neuen Datensatz
  }
  if ($TagArt == 2) # Way Tag
  { 
  	$ret = $DSW->CrunchLine($Zeile); # Way Tags abarbeiten
  	if ($ret == 99) { $TagArt = 0; } # 99 => Datensatz wurde in die Datenbank geschrieben, bereit fuer neuen Datensatz
  }
  if ($TagArt == 3) # Rel Tag
  { 
  open AD, ">>err.log";
    print AD "REF Tag gefunden: $Zeile";
  close AD;
  }
  if ($TagArt == 4) # Relation Tag
  { 
  	$ret = $DSR->CrunchLine($Zeile); # Way Tags abarbeiten
  	if ($ret == 99) { $TagArt = 0; } # 99 => Datensatz wurde in die Datenbank geschrieben, bereit fuer neuen Datensatz    
  }
  
  if ($ret ==  0) { print "Nicht verarbeitete Zeile: $Zeile"; }  
####
# Fortschrittsanzeige - alle 5000 Zeilen eine Ausgabe
####  
  $Zeilencount ++;
  if ($Zeilencount % 5000 == 0)
  {
    @mTime = localtime(time);
    if (length($mTime[0]) == 1) { $mTime[0] = "0" . $mTime[0]; }
    if (length($mTime[1]) == 1) { $mTime[1] = "0" . $mTime[1]; }
    if (length($mTime[2]) == 1) { $mTime[2] = "0" . $mTime[2]; }

    $tStr  = $mTime[2] . ":" . $mTime[1] . ":" . $mTime[0];
    $perc = sprintf("%0.5f", 100/$TotalBytes*$Abgearbeitet);
    print "$tStr  $Abgearbeitet von $TotalBytes bytes erledigt. ($perc %)\n";
  }
} # Ende while Schleife

####
# Am Schluss nochmal eine Fortschrittsanzeige
####
print  "$TotalBytes von $TotalBytes bytes erledigt. (100 %)\n";
print  "Datenimport vollstaendig\n";
print  "Erstelle Inhaltsverzeichnis...\n";
$DBC->CreateIndices($Databasename);

exit;
sub PruefeAPIVersion
{
  while (<$source>)
  {
    if ($_ =~ /<osm version=\"0.6\"/) { return 1; }
  }
  die("Konnte API Version 0.6 nicht feststellen\n");
}
